@@ -8,6 +8,7 @@ Translates from the [numpydoc docstring format](https://numpydoc.readthedocs.io/
 
 Translates from [Google's docstring format](https://google.github.io/styleguide/pyguide.html)
 """
+
 import ast
 from ast import AST
 from collections import OrderedDict
@@ -92,12 +93,7 @@ def parse_docstring(
     assert isinstance(docstring, (type(None), str)), "{typ} != str".format(
         typ=type(docstring).__name__
     )
-    if docstring is None or any(map(partial(contains, docstring), TOKENS.rest)):
-        style = Style.rest
-    elif any(map(partial(contains, docstring), TOKENS.google)):
-        style = Style.google
-    else:
-        style = Style.numpydoc
+    style = derive_docstring_format(docstring)
 
     ir = {
         "name": None,
@@ -144,6 +140,25 @@ def parse_docstring(
     return ir
 
 
+def derive_docstring_format(docstring):
+    """
+    Infer the docstring format of the provided docstring
+
+    :param docstring: the docstring
+    :type docstring: ```Optional[str]```
+
+    :returns: the style of docstring
+    :rtype: ```Literal['rest', 'numpydoc', 'google']```
+    """
+    if docstring is None or any(map(partial(contains, docstring), TOKENS.rest)):
+        style = Style.rest
+    elif any(map(partial(contains, docstring), TOKENS.google)):
+        style = Style.google
+    else:
+        style = Style.numpydoc
+    return style
+
+
 def _scan_phase(docstring, parse_original_whitespace=False, style=Style.rest):
     """
     Scanner phase. Lexical analysis; to some degree…
@@ -481,7 +496,7 @@ def _set_name_and_type(param, infer_type, word_wrap):
     google_opt = ", optional"
     if _param.get("typ"):
         _param["typ"] = (
-            "Optional[{}]".format(_param["typ"][: -len(google_opt)])
+            "Optional[{typ}]".format(typ=_param["typ"][: -len(google_opt)])
             if _param["typ"].endswith(google_opt)
             else _param["typ"]
         )
@@ -507,7 +522,7 @@ def _set_name_and_type(param, infer_type, word_wrap):
             and "typ" in _param
             and not _param["typ"].startswith("Optional[")
         ):
-            _param["typ"] = "Optional[{}]".format(_param["typ"])
+            _param["typ"] = "Optional[{typ}]".format(typ=_param["typ"])
 
     return name, _param
 
@@ -653,16 +668,20 @@ def _parse_phase_numpydoc_and_google(
             if typ:
                 assert typ.startswith("(") and typ.endswith(
                     ")"
-                ), "Expected third partition to be paren wrapped {!r}".format(s)
+                ), "Expected third partition to be paren wrapped {s!r}".format(s=s)
                 cur["typ"] = typ[1:-1]
                 if " or " in cur["typ"]:
-                    cur["typ"] = "Union[{}]".format(", ".join(cur["typ"].split(" or ")))
+                    cur["typ"] = "Union[{types}]".format(
+                        types=", ".join(cur["typ"].split(" or "))
+                    )
                 end = white_spacer(scan[0][offset + 1 :])
                 if len(end) > 3 and end.startswith("{") and end.endswith("}"):
                     # PyTorch invented their own syntax for this I guess?
                     cur["typ"], scan[0] = (
-                        "Literal{}".format(
-                            list(map(rpartial(str.strip, "'"), end[1:-1].split(", ")))
+                        "Literal{literal_type_list}".format(
+                            literal_type_list=list(
+                                map(rpartial(str.strip, "'"), end[1:-1].split(", "))
+                            )
                         ),
                         "",
                     )
@@ -691,7 +710,9 @@ def _parse_phase_numpydoc_and_google(
             scanned_params[:afterward_idx],
             scanned_params[afterward_idx:],
         )
-        scanned["doc"] += "\n\n{}".format("\n".join(map("\n".join, scanned_afterward)))
+        scanned["doc"] += "\n\n{afterward}".format(
+            afterward="\n".join(map("\n".join, scanned_afterward))
+        )
 
     if "scanned_afterward" in scanned:
         _fill_doc_with_afterward(scanned)
@@ -949,4 +970,4 @@ def _set_param_values(input_str, val, sw=":type"):
     )
 
 
-__all__ = ["parse_docstring"]
+__all__ = ["derive_docstring_format", "parse_docstring"]
