@@ -1,6 +1,7 @@
 """
 Tests for the Intermediate Representation produced by the parsers
 """
+
 import ast
 from ast import FunctionDef
 from collections import OrderedDict
@@ -381,7 +382,7 @@ class TestParsers(TestCase):
                         (
                             "replaced",
                             {
-                                "doc": "whether a node has been replaced "
+                                "doc": "Whether a node has been replaced "
                                 "(only replaces first "
                                 "occurrence)"
                             },
@@ -409,6 +410,7 @@ class TestParsers(TestCase):
             class_torch_nn_l1loss_ast,
             merge_inner_function="__init__",
             infer_type=True,
+            parse_original_whitespace=True,
         )
 
         del parsed_ir["_internal"]  # Not needed for this test
